@@ -257,7 +257,7 @@ ok:
 		}
 	}
 	if(verbose>=0){
-		char *s = "$Id: comedi_calibrate.c,v 1.89 2003/08/08 17:38:59 fmhess Exp $";
+		char *s = "$Id: comedi_calibrate.c,v 1.92 2004/03/29 01:34:15 fmhess Exp $";
 
 		printf("%.*s\n",(int)strlen(s)-2,s+1);
 		printf("Driver name: %s\n", options.driver_name);
@@ -319,7 +319,7 @@ ok:
 	return retval;
 }
 
-void set_target( calibration_setup_t *setup, int obs,double target)
+void set_target( calibration_setup_t *setup, int obs, double target)
 {
 	comedi_range *range;
 	lsampl_t maxdata, data;
@@ -397,19 +397,22 @@ int preobserve( calibration_setup_t *setup, int obs)
 	lsampl_t ref_data[ 2 ];
 
 	// setup reference source
-	memset( &reference_source_config, 0, sizeof(reference_source_config) );
-	reference_source_config.insn = INSN_CONFIG;
-	reference_source_config.n = 2;
-	reference_source_config.subdev = setup->ad_subdev;
-	reference_source_config.data = ref_data;
-	reference_source_config.data[ 0 ] = INSN_CONFIG_ALT_SOURCE;
-	reference_source_config.data[ 1 ] = setup->observables[obs].reference_source;
+	if(setup->observables[obs].reference_source >= 0)
+	{
+		memset( &reference_source_config, 0, sizeof(reference_source_config) );
+		reference_source_config.insn = INSN_CONFIG;
+		reference_source_config.n = 2;
+		reference_source_config.subdev = setup->ad_subdev;
+		reference_source_config.data = ref_data;
+		reference_source_config.data[ 0 ] = INSN_CONFIG_ALT_SOURCE;
+		reference_source_config.data[ 1 ] = setup->observables[obs].reference_source;
 
-	retval = comedi_do_insn( setup->dev, &reference_source_config );
-	/* ignore errors for now since older ni driver doesn't
-	 * support reference config insn */
-	if( retval < 0 )
-		perror("preobserve() ignoring reference config error" );
+		retval = comedi_do_insn( setup->dev, &reference_source_config );
+		/* ignore errors for now since older ni driver doesn't
+		* support reference config insn */
+		if( retval < 0 )
+			perror("preobserve() ignoring reference config error" );
+	}
 	retval = 0;
 
 	if( setup->observables[obs].preobserve_insn.n != 0){
@@ -914,14 +917,16 @@ void setup_caldacs( calibration_setup_t *setup, int caldac_subdev )
 	}
 
 	// XXX check subdevice type is really calibration
-	// XXX check we dont exceed max number of allowable caldacs
 
 	n_chan = comedi_get_n_channels( setup->dev, caldac_subdev );
+	assert(n_chan >= 0);
+	assert(setup->n_caldacs + n_chan < N_CALDACS);
 
 	for(i = 0; i < n_chan; i++){
 		setup->caldacs[ setup->n_caldacs + i ].subdev = caldac_subdev;
 		setup->caldacs[ setup->n_caldacs + i ].chan = i;
-		setup->caldacs[ setup->n_caldacs + i ].maxdata = comedi_get_maxdata( setup->dev, caldac_subdev, i);
+		setup->caldacs[ setup->n_caldacs + i ].maxdata = comedi_get_maxdata(setup->dev, caldac_subdev, i);
+		assert(setup->caldacs[ setup->n_caldacs + i ].maxdata > 0);
 		setup->caldacs[ setup->n_caldacs + i ].current=0;
 	}
 
@@ -1177,6 +1182,7 @@ int get_bipolar_lowgain(comedi_t *dev,int subdev)
 	for(i=0;i<n_ranges;i++){
 		range = comedi_get_range(dev,subdev,0,i);
 		if( is_bipolar( dev, subdev, 0, i ) == 0 ) continue;
+		if(range->unit == UNIT_none ) continue;
 		if(range->max>max){
 			ret = i;
 			max=range->max;
@@ -1197,6 +1203,7 @@ int get_bipolar_highgain(comedi_t *dev,int subdev)
 	for(i=0;i<n_ranges;i++){
 		range = comedi_get_range(dev,subdev,0,i);
 		if( is_bipolar( dev, subdev, 0, i ) == 0 ) continue;
+		if(range->unit == UNIT_none ) continue;
 		if(range->max<min){
 			ret = i;
 			min=range->max;
@@ -1217,6 +1224,7 @@ int get_unipolar_lowgain(comedi_t *dev,int subdev)
 	for(i=0;i<n_ranges;i++){
 		range = comedi_get_range(dev,subdev,0,i);
 		if( is_unipolar( dev, subdev, 0, i ) == 0 ) continue;
+		if(range->unit == UNIT_none ) continue;
 		if(range->max>max){
 			ret = i;
 			max=range->max;
@@ -1237,6 +1245,7 @@ int get_unipolar_highgain(comedi_t *dev,int subdev)
 	for(i=0;i<n_ranges;i++){
 		range = comedi_get_range(dev,subdev,0,i);
 		if( is_unipolar( dev, subdev, 0, i ) == 0 ) continue;
+		if(range->unit == UNIT_none ) continue;
 		if(range->max < max){
 			ret = i;
 			max=range->max;
@@ -1457,7 +1466,7 @@ int linear_fit_monotonic(linear_fit_t *l)
 		l->sxx+=x*x;
 	}
 	sxp=l->sxx-l->sx*l->sx/l->s1;
-	
+
 	l->ave_x=l->sx/l->s1;
 	l->ave_y=l->sy/l->s1;
 	l->slope=(l->s1*l->sxy-l->sx*l->sy)/(l->s1*l->sxx-l->sx*l->sx);
