@@ -33,7 +33,7 @@
 #include "calib.h"
 
 
-char ni_id[] = "$Id: ni.c,v 1.98 2003/08/02 03:20:22 fmhess Exp $";
+char ni_id[] = "$Id: ni.c,v 1.130 2004/09/09 21:43:16 fmhess Exp $";
 
 struct board_struct{
 	char *name;
@@ -47,14 +47,18 @@ struct board_struct{
 static int ni_setup_board( calibration_setup_t *setup , const char *device_name );
 static void ni_setup_observables( calibration_setup_t *setup );
 static void ni_setup_observables_611x( calibration_setup_t *setup );
+static void ni67xx_setup_observables( calibration_setup_t *setup );
 
 static int cal_ni_at_mio_16e_2(calibration_setup_t *setup);
+static int cal_ni_at_mio_64e_3(calibration_setup_t *setup);
 static int cal_ni_daqcard_ai_16xe_50(calibration_setup_t *setup);
 static int cal_ni_at_mio_16e_1(calibration_setup_t *setup);
 static int cal_ni_pci_mio_16e_1(calibration_setup_t *setup);
+static int cal_ni_pci_6014(calibration_setup_t *setup);
 static int cal_ni_pci_6024e(calibration_setup_t *setup);
 static int cal_ni_pci_6025e(calibration_setup_t *setup);
 static int cal_ni_pci_6032e(calibration_setup_t *setup);
+static int cal_ni_pci_6034e(calibration_setup_t *setup);
 static int cal_ni_pci_6035e(calibration_setup_t *setup);
 static int cal_ni_pci_6036e(calibration_setup_t *setup);
 static int cal_ni_pci_6071e(calibration_setup_t *setup);
@@ -70,32 +74,37 @@ static int cal_ni_pci_611x(calibration_setup_t *setup);
 static int cal_ni_pci_mio_16e_4(calibration_setup_t *setup);
 static int cal_ni_daqcard_6062e(calibration_setup_t *setup);
 static int cal_ni_daqcard_6024e(calibration_setup_t *setup);
+static int cal_ni_daqcard_6036e(calibration_setup_t *setup);
+static int cal_ni_pci_6711(calibration_setup_t *setup);
 
 static double ni_get_reference( calibration_setup_t *setup, int lsb_loc,int msb_loc);
 
 static struct board_struct boards[]={
 	{ "at-ai-16xe-10", STATUS_UNKNOWN, NULL, ni_setup_observables, 0x1b7, 0x1b8 },
 	{ "at-mio-16de-10", STATUS_UNKNOWN, NULL, ni_setup_observables, 0x1a7, 0x1a8 },
-	{ "at-mio-16e-1", STATUS_SOME, cal_ni_at_mio_16e_1, ni_setup_observables, 0x1a9, 0x1aa },
+	{ "at-mio-16e-1", STATUS_DONE, cal_ni_at_mio_16e_1, ni_setup_observables, 0x1a9, 0x1aa },
 	{ "at-mio-16e-2", STATUS_DONE, cal_ni_at_mio_16e_2, ni_setup_observables, 0x1a9, 0x1aa },
-	{ "at-mio-16e-10", STATUS_GUESS, cal_ni_at_mio_16e_10, ni_setup_observables, 0x1a7, 0x1a8 },
+	{ "at-mio-16e-10", STATUS_DONE, cal_ni_at_mio_16e_10, ni_setup_observables, 0x1a7, 0x1a8 },
 	{ "at-mio-16xe-10", STATUS_UNKNOWN, NULL, ni_setup_observables, 0x1b7, 0x1b8 },
 	{ "at-mio-16xe-50", STATUS_SOME, cal_ni_at_mio_16xe_50, ni_setup_observables, 0x1b5, 0x1b6 },
+	{ "at-mio-64e-3", STATUS_SOME, cal_ni_at_mio_64e_3, ni_setup_observables, 0x1a9, 0x1aa},
 	{ "DAQCard-ai-16e-4", STATUS_DONE, cal_ni_daqcard_ai_16e_4, ni_setup_observables, 0x1b5, 0x1b6 },
 	{ "DAQCard-ai-16xe-50", STATUS_DONE, cal_ni_daqcard_ai_16xe_50, ni_setup_observables, 0x1be, 0x1bf },
 	{ "DAQCard-6024E", STATUS_SOME, cal_ni_daqcard_6024e, ni_setup_observables, -1, -1 },
+	{ "DAQCard-6036E", STATUS_DONE, cal_ni_daqcard_6036e, ni_setup_observables, 0x1ab, 0x1ac },
 	{ "DAQCard-6062E", STATUS_DONE, cal_ni_daqcard_6062e, ni_setup_observables, 0x1a9, 0x1aa },
 	{ "pci-mio-16e-1", STATUS_DONE, cal_ni_pci_mio_16e_1, ni_setup_observables, 0x1a9, 0x1aa },
 	{ "pci-mio-16e-4", STATUS_SOME, cal_ni_pci_mio_16e_4, ni_setup_observables, 0x1a9, 0x1aa },
 	{ "pci-mio-16xe-10", STATUS_DONE,	cal_ni_pci_mio_16xe_10,	ni_setup_observables, 0x1ae, 0x1af },
 	{ "pci-mio-16xe-50", STATUS_SOME, cal_ni_pci_mio_16xe_50, ni_setup_observables, 0x1b5, 0x1b6 },
+	{ "pci-6014", STATUS_DONE, cal_ni_pci_6014, ni_setup_observables, 0x1ab, 0x1ac },
 	{ "pci-6023e", STATUS_DONE, cal_ni_pci_6023e, ni_setup_observables, 0x1bb, 0x1bc },
 	{ "pci-6024e", STATUS_SOME, cal_ni_pci_6024e, ni_setup_observables, 0x1af, 0x1b0 },
 	{ "pci-6025e", STATUS_SOME, cal_ni_pci_6025e, ni_setup_observables, 0x1af, 0x1b0 },
 	{ "pci-6031e", STATUS_DONE, cal_ni_pci_mio_16xe_10, ni_setup_observables, 0x1ae, 0x1af },
 	{ "pci-6032e", STATUS_DONE, cal_ni_pci_6032e, ni_setup_observables, 0x1ae, 0x1af },
-	{ "pci-6033e", STATUS_UNKNOWN, NULL, ni_setup_observables, 0x1ae, 0x1af },
-	{ "pci-6034e", STATUS_UNKNOWN, NULL, ni_setup_observables, -1, -1 },
+	{ "pci-6033e", STATUS_DONE, cal_ni_pci_6032e, ni_setup_observables, 0x1b7, 0x1b8 },
+	{ "pci-6034e", STATUS_DONE, cal_ni_pci_6034e, ni_setup_observables, 0x1bb, 0x1bc },
 	{ "pci-6035e", STATUS_DONE, cal_ni_pci_6035e, ni_setup_observables, 0x1af, 0x1b0 },
 	{ "pci-6036e", STATUS_DONE, cal_ni_pci_6036e, ni_setup_observables, 0x1ab, 0x1ac },
 	{ "pci-6052e", STATUS_DONE, cal_ni_pci_6052e, ni_setup_observables, 0x19f, 0x1a0 },
@@ -109,10 +118,16 @@ static struct board_struct boards[]={
 	{ "pxi-6052e", STATUS_UNKNOWN, NULL, ni_setup_observables, -1, -1 },
 	{ "pxi-6070e", STATUS_UNKNOWN, NULL, ni_setup_observables, -1, -1 },
 	{ "pxi-6071e", STATUS_GUESS, cal_ni_pxi_6071e, ni_setup_observables, -1, -1 },
+	{ "pci-6711", STATUS_DONE, cal_ni_pci_6711, ni67xx_setup_observables, 0x1d4, 0x1d5},
+	{ "pci-6713", STATUS_DONE, cal_ni_pci_6711, ni67xx_setup_observables, 0x1d4, 0x1d5},
+	{ "pci-6731", STATUS_GUESS, cal_ni_pci_6711, ni67xx_setup_observables, -1, -1},
+	{ "pci-6733", STATUS_GUESS, cal_ni_pci_6711, ni67xx_setup_observables, -1, -1},
+	{ "pxi-6711", STATUS_GUESS, cal_ni_pci_6711, ni67xx_setup_observables, -1, -1},
+	{ "pxi-6713", STATUS_GUESS, cal_ni_pci_6711, ni67xx_setup_observables, -1, -1},
+	{ "pxi-6731", STATUS_GUESS, cal_ni_pci_6711, ni67xx_setup_observables, -1, -1},
+	{ "pxi-6733", STATUS_GUESS, cal_ni_pci_6711, ni67xx_setup_observables, -1, -1},
 #if 0
 	{ "at-mio-64e-3",	cal_ni_16e_1 },
-	{ "pci-6711",		cal_ni_unknown },
-	{ "pci-6713",		cal_ni_unknown },
 #endif
 };
 #define n_boards (sizeof(boards)/sizeof(boards[0]))
@@ -233,6 +248,7 @@ typedef struct
 	int adc_unip_offset;
 	int adc_unip_offset_fine;
 	int dac_offset[ 2 ];
+	int dac_offset_fine[ 2 ];
 	int dac_gain[ 2 ];
 	int dac_gain_fine[ 2 ];
 	int dac_linearity[ 2 ];
@@ -256,6 +272,7 @@ static inline void init_ni_caldac_layout( ni_caldac_layout_t *layout )
 	for( i = 0; i < 2; i++ )
 	{
 		layout->dac_offset[ i ] = -1;
+		layout->dac_offset_fine[ i ] = -1;
 		layout->dac_gain[ i ] = -1;
 		layout->dac_gain_fine[ i ] = -1;
 		layout->dac_linearity[ i ] = -1;
@@ -300,7 +317,9 @@ static void ni_setup_ao_observables( calibration_setup_t *setup )
 	int ao_unipolar_lowgain;
 
 	ai_bipolar_lowgain = get_bipolar_lowgain( setup->dev, setup->ad_subdev);
+	assert(ai_bipolar_lowgain >= 0);
 	ao_bipolar_lowgain = get_bipolar_lowgain( setup->dev, setup->da_subdev);
+	assert(ao_bipolar_lowgain >= 0);
 	ao_unipolar_lowgain = get_unipolar_lowgain( setup->dev, setup->da_subdev);
 
 	memset(&tmpl,0,sizeof(tmpl));
@@ -647,23 +666,6 @@ static void ni_setup_observables_611x( calibration_setup_t *setup )
 	setup->n_observables = num_ao_observables_611x + 2 * num_ai_ranges * num_ai_channels;
 }
 
-static int cal_ni_at_mio_16e_2(calibration_setup_t *setup)
-{
-	ni_caldac_layout_t layout;
-
-	init_ni_caldac_layout( &layout );
-	layout.adc_pregain_offset = 0;
-	layout.adc_postgain_offset = 1;
-	layout.adc_gain = 3;
-	layout.adc_unip_offset = 2;
-	layout.dac_offset[ 0 ] = 5;
-	layout.dac_gain[ 0 ] = 6;
-	layout.dac_offset[ 1 ] = 8;
-	layout.dac_gain[ 1 ] = 9;
-
-	return cal_ni_generic( setup, &layout );
-}
-
 static int cal_ni_daqcard_ai_16xe_50(calibration_setup_t *setup)
 {
 	ni_caldac_layout_t layout;
@@ -714,7 +716,26 @@ static int cal_ni_pci_mio_16xe_10(calibration_setup_t *setup)
 
 static int cal_ni_at_mio_16e_1(calibration_setup_t *setup)
 {
-	return cal_ni_at_mio_16e_2( setup );
+	ni_caldac_layout_t layout;
+
+	init_ni_caldac_layout( &layout );
+	layout.adc_pregain_offset = 0;
+	layout.adc_postgain_offset = 1;
+	layout.adc_gain = 3;
+	layout.adc_unip_offset = 2;
+	layout.dac_offset[0] = 5;
+	layout.dac_gain[0] = 6;
+	layout.dac_linearity[0] = 4;
+	layout.dac_offset[1] = 8;
+	layout.dac_gain[1] = 9;
+	layout.dac_linearity[1] = 7;
+
+	return cal_ni_generic( setup, &layout );
+}
+
+static int cal_ni_at_mio_16e_2(calibration_setup_t *setup)
+{
+	return cal_ni_at_mio_16e_1(setup);
 }
 
 static int cal_ni_pci_mio_16e_1(calibration_setup_t *setup)
@@ -736,6 +757,26 @@ static int cal_ni_pci_mio_16e_1(calibration_setup_t *setup)
 	return cal_ni_generic( setup, &layout );
 }
 
+static int cal_ni_pci_6014(calibration_setup_t *setup)
+{
+	ni_caldac_layout_t layout;
+
+	init_ni_caldac_layout( &layout );
+	layout.adc_pregain_offset = 0;
+	layout.adc_postgain_offset = 4;
+	layout.adc_pregain_offset_fine = 8;
+	layout.adc_gain = 2;
+	layout.dac_offset[0] = 6;
+	layout.dac_offset_fine[0] = 10;
+	layout.dac_gain[0] = 7;
+	layout.dac_gain_fine[0] = 11;
+	layout.dac_offset[1] = 9;
+	layout.dac_offset_fine[1] = 1;
+	layout.dac_gain[1] = 3;
+	layout.dac_gain_fine[1] = 5;
+	return cal_ni_generic( setup, &layout );
+}
+
 static int cal_ni_pci_6032e(calibration_setup_t *setup)
 {
 	ni_caldac_layout_t layout;
@@ -750,6 +791,19 @@ static int cal_ni_pci_6032e(calibration_setup_t *setup)
 	return cal_ni_generic( setup, &layout );
 }
 
+static int cal_ni_pci_6034e(calibration_setup_t *setup)
+{
+	ni_caldac_layout_t layout;
+
+	init_ni_caldac_layout( &layout );
+	layout.adc_pregain_offset = 0;
+	layout.adc_pregain_offset_fine = 8;
+	layout.adc_postgain_offset = 4;
+	layout.adc_gain = 2;
+
+	return cal_ni_generic( setup, &layout );
+}
+
 static int cal_ni_pci_6035e(calibration_setup_t *setup)
 {
 	/* this is for the ad8804_debug caldac */
@@ -848,20 +902,25 @@ static int cal_ni_pxi_6071e(calibration_setup_t *setup)
 
 static int cal_ni_at_mio_16e_10(calibration_setup_t *setup)
 {
-	// 16e-10 (old)
 	ni_caldac_layout_t layout;
 
+	if(comedi_get_version_code(setup->dev) <= COMEDI_VERSION_CODE(0, 7, 68))
+	{
+		DPRINT(0, "WARNING: you need comedi driver version 0.7.69 or later\n"
+		 "for this calibration to work properly\n" );
+	}
 	init_ni_caldac_layout( &layout );
-	layout.adc_pregain_offset = 10;
-	layout.adc_pregain_offset_fine = 0;
-	layout.adc_postgain_offset = 1;
-	layout.adc_gain = 3;
-	layout.adc_unip_offset = 2;
-	layout.dac_offset[ 0 ] = 5; /* guess */
-	layout.dac_gain[ 0 ] = 6; /* guess */
-	layout.dac_offset[ 1 ] = 8; /* guess */
-	layout.dac_gain[ 1 ] = 9; /* guess */
-
+	layout.adc_pregain_offset = 0;
+	layout.adc_pregain_offset_fine = 8;
+	layout.adc_postgain_offset = 4;
+	layout.adc_gain = 2;
+	layout.adc_unip_offset = 7;
+	layout.dac_offset[ 0 ] = 6;
+	layout.dac_gain[ 0 ] = 11; 
+	layout.dac_linearity[ 0 ] = 10;
+	layout.dac_offset[ 1 ] = 9;
+	layout.dac_gain[ 1 ] = 5;
+	layout.dac_linearity[1] = 1;
 	return cal_ni_generic( setup, &layout );
 }
 
@@ -902,9 +961,9 @@ static int cal_ni_pci_6024e(calibration_setup_t *setup)
 	ni_caldac_layout_t layout;
 
 	init_ni_caldac_layout( &layout );
-	layout.adc_pregain_offset = 8;
+	layout.adc_pregain_offset = 0;
 	layout.adc_postgain_offset = 4;
-	layout.adc_pregain_offset_fine = 0;
+	layout.adc_pregain_offset_fine = 8;
 	layout.adc_gain = 2;
 	layout.dac_offset[ 0 ] = 6;
 	layout.dac_gain[ 0 ] = 11;
@@ -921,9 +980,9 @@ static int cal_ni_pci_6025e(calibration_setup_t *setup)
 	ni_caldac_layout_t layout;
 
 	init_ni_caldac_layout( &layout );
-	layout.adc_pregain_offset = 8;
+	layout.adc_pregain_offset = 0;
 	layout.adc_postgain_offset = 4;
-	layout.adc_pregain_offset_fine = 0;
+	layout.adc_pregain_offset_fine = 8;
 	layout.adc_gain = 2;
 	layout.dac_offset[ 0 ] = 6;
 	layout.dac_gain[ 0 ] = 11;
@@ -1113,6 +1172,63 @@ static int cal_ni_daqcard_6024e( calibration_setup_t *setup )
 	return cal_ni_generic( setup, &layout );
 }
 
+static int cal_ni_daqcard_6036e( calibration_setup_t *setup )
+{
+	ni_caldac_layout_t layout;
+
+	if( comedi_get_version_code( setup->dev ) <= COMEDI_VERSION_CODE( 0, 7, 68 ) )
+	{
+		DPRINT(0, "WARNING: you need comedi driver version 0.7.69 or later\n"
+		 "for this calibration to work properly\n" );
+	}
+
+	init_ni_caldac_layout( &layout );
+
+	layout.adc_pregain_offset = 0;
+	layout.adc_pregain_offset_fine = 8;
+	layout.adc_postgain_offset = 4;
+	layout.adc_gain = 2;
+
+	layout.dac_offset[0] = 6;
+	layout.dac_gain[0] = 7;
+	layout.dac_gain_fine[ 0 ] = 11;
+	layout.dac_linearity[0] = 10;
+
+	layout.dac_offset[ 1 ] = 9;
+	layout.dac_gain[ 1 ] = 3;
+	layout.dac_gain_fine[ 1 ] = 5;
+	layout.dac_linearity[1] = 1;
+
+	return cal_ni_generic( setup, &layout );
+}
+
+static int cal_ni_at_mio_64e_3( calibration_setup_t *setup )
+{
+	ni_caldac_layout_t layout;
+
+	if( comedi_get_version_code( setup->dev ) <= COMEDI_VERSION_CODE( 0, 7, 68 ) )
+	{
+		DPRINT(0, "WARNING: you need comedi driver version 0.7.69 or later\n"
+		 "for this calibration to work properly\n" );
+	}
+
+	init_ni_caldac_layout( &layout );
+	
+	layout.adc_pregain_offset = 8;
+	layout.adc_postgain_offset = 4;
+	layout.adc_gain = 2;
+
+	layout.dac_offset[0] = 6;
+	layout.dac_gain[0] = 14;
+	layout.dac_linearity[0] = 10;
+
+	layout.dac_offset[ 1 ] = 9;
+	layout.dac_gain[ 1 ] = 5;
+	layout.dac_linearity[1] = 1;
+
+	return cal_ni_generic( setup, &layout );
+}
+
 static void prep_adc_caldacs_generic( calibration_setup_t *setup,
 	const ni_caldac_layout_t *layout, unsigned int range )
 {
@@ -1157,6 +1273,7 @@ static void prep_dac_caldacs_generic( calibration_setup_t *setup,
 	if( setup->old_calibration == NULL )
 	{
 		reset_caldac( setup, layout->dac_offset[ channel ] );
+		reset_caldac( setup, layout->dac_offset_fine[ channel ] );
 		reset_caldac( setup, layout->dac_gain[ channel ] );
 		reset_caldac( setup, layout->dac_gain_fine[ channel ] );
 		reset_caldac( setup, layout->dac_linearity[ channel ] );
@@ -1168,6 +1285,7 @@ static void prep_dac_caldacs_generic( calibration_setup_t *setup,
 		{
 			DPRINT( 0, "Failed to apply existing calibration, reseting dac caldacs.\n" );
 			reset_caldac( setup, layout->dac_offset[ channel ] );
+			reset_caldac( setup, layout->dac_offset_fine[ channel ] );
 			reset_caldac( setup, layout->dac_gain[ channel ] );
 			reset_caldac( setup, layout->dac_gain_fine[ channel ] );
 			reset_caldac( setup, layout->dac_linearity[ channel ] );
@@ -1247,20 +1365,21 @@ static int cal_ni_generic( calibration_setup_t *setup, const ni_caldac_layout_t
 				layout->adc_unip_offset );
 			generic_do_cal( setup, current_cal, ni_unip_zero_offset_high,
 				layout->adc_unip_offset_fine );
+		/* if we don't have a unipolar offset caldac, do a fully
+		 * independent calibration for unipolar ranges */
 		}else
 		{
 			prep_adc_caldacs_generic( setup, layout, ai_unipolar_lowgain );
 			generic_peg( setup, ni_unip_zero_offset_low,
 				layout->adc_pregain_offset, 1 );
-			reset_caldac( setup, layout->adc_gain_fine );
+			generic_peg( setup, ni_unip_zero_offset_low,
+				layout->adc_postgain_offset, 1 );
 			generic_do_relative( setup, current_cal, ni_unip_zero_offset_low,
 				ni_unip_reference_low, layout->adc_gain );
-			reset_caldac( setup, layout->adc_postgain_offset_fine );
 			generic_do_relative( setup, current_cal, ni_unip_zero_offset_low,
 				ni_unip_zero_offset_high, layout->adc_postgain_offset );
 			generic_do_relative( setup, current_cal, ni_unip_zero_offset_low,
 				ni_unip_zero_offset_high, layout->adc_postgain_offset_fine );
-			reset_caldac( setup, layout->adc_pregain_offset_fine );
 			generic_do_cal( setup, current_cal, ni_unip_zero_offset_high,
 				layout->adc_pregain_offset );
 			generic_do_relative( setup, current_cal, ni_unip_zero_offset_low,
@@ -1294,8 +1413,11 @@ static int cal_ni_generic( calibration_setup_t *setup, const ni_caldac_layout_t
 			generic_do_linearity( setup, current_cal, ni_ao_zero_offset( channel ),
 				ni_ao_mid_linearity( channel ), ni_ao_reference( channel ),
 				layout->dac_linearity[ channel ] );
+			reset_caldac(setup, layout->dac_offset_fine[channel]);
 			generic_do_cal( setup, current_cal, ni_ao_zero_offset( channel ),
 				layout->dac_offset[ channel ] );
+			generic_do_cal( setup, current_cal, ni_ao_zero_offset( channel ),
+				layout->dac_offset_fine[ channel ] );
 			reset_caldac( setup, layout->dac_gain_fine[ channel ] );
 			generic_do_cal( setup, current_cal, ni_ao_reference( channel ),
 				layout->dac_gain[ channel ] );
@@ -1318,8 +1440,12 @@ static int cal_ni_generic( calibration_setup_t *setup, const ni_caldac_layout_t
 				generic_do_linearity( setup, current_cal, ni_ao_unip_low_linearity( channel ),
 					ni_ao_unip_mid_linearity( channel ), ni_ao_unip_reference( channel ),
 					layout->dac_linearity[ channel ] );
+				reset_caldac( setup, layout->dac_offset_fine[ channel ] );
 				generic_do_cal( setup, current_cal, ni_ao_unip_zero_offset( channel),
 					layout->dac_offset[ channel ] );
+				generic_do_cal( setup, current_cal, ni_ao_unip_zero_offset( channel),
+					layout->dac_offset_fine[ channel ] );
+				reset_caldac( setup, layout->dac_gain_fine[ channel ] );
 				generic_do_cal( setup, current_cal, ni_ao_unip_reference( channel ),
 					layout->dac_gain[ channel ] );
 				generic_do_cal( setup, current_cal, ni_ao_unip_reference( channel ),
@@ -1361,3 +1487,204 @@ static double ni_get_reference( calibration_setup_t *setup, int lsb_loc,int msb_
 	return ref;
 }
 
+/****************
+ NI 671x and 673x support
+ **************/
+
+static const int channels_per_ad8804 = 16;
+
+static inline int ni67xx_ao_gain_caldac(unsigned int ao_channel)
+{
+	int ad8804_gain_channels[4] = {8, 2, 11, 5};
+	int caldac_channel = ad8804_gain_channels[ao_channel % 4];
+	int caldac_index = ao_channel / 4;
+	/* just guessing that second ad8804 is works for ao channels 4-7
+	 * the same as the first ad8804 works for ao channels 0-3 */
+	return caldac_index * channels_per_ad8804 + caldac_channel;
+}
+static inline int ni67xx_ao_linearity_caldac(unsigned int ao_channel)
+{
+	int ad8804_linearity_channels[4] = {4, 10, 1, 0};
+	int caldac_channel = ad8804_linearity_channels[ao_channel % 4];
+	int caldac_index = ao_channel / 4;
+
+	return caldac_index * channels_per_ad8804 + caldac_channel;
+}
+static inline int ni67xx_ao_offset_caldac(unsigned int ao_channel)
+{
+	int ad8804_offset_channels[4] = {7, 6, 9, 3};
+	int caldac_channel = ad8804_offset_channels[ao_channel % 4];
+	int caldac_index = ao_channel / 4;
+
+	return caldac_index * channels_per_ad8804 + caldac_channel;
+}
+
+static int ni67xx_ao_ground_observable_index( const calibration_setup_t *setup,
+	unsigned int channel, unsigned int ao_range )
+{
+	return 3 * channel + 0;
+}
+
+static int ni67xx_ao_mid_observable_index( const calibration_setup_t *setup,
+	unsigned int channel, unsigned int ao_range )
+{
+	return 3 * channel + 1;
+}
+
+static int ni67xx_ao_high_observable_index( const calibration_setup_t *setup,
+	unsigned int channel, unsigned int ao_range )
+{
+	return 3 * channel + 2;
+}
+
+static const double ni67xx_unitless_adc_offset = 0.5;
+
+/* determine conversion factor between actual voltage and
+ * interval [0,1) returned by reads from the calibration adc
+ * subdevice.
+ */
+static double ni67xx_unitless_adc_slope(calibration_setup_t *setup)
+{
+	double reference_in_volts;
+	double reference_unitless;
+	double slope;
+	comedi_insn insn;
+	lsampl_t data;
+	comedi_range *range;
+	static const int maxdata = 0x10000;
+	int retval;
+
+	if(ni_board(setup)->ref_eeprom_lsb >= 0 &&
+		ni_board(setup)->ref_eeprom_msb >= 0)
+	{
+		reference_in_volts = ni_get_reference(setup,
+			ni_board(setup)->ref_eeprom_lsb, ni_board(setup)->ref_eeprom_msb );
+	}else
+	{
+		DPRINT( 0, "WARNING: unknown eeprom address for reference voltage\n"
+			"correction.  This might be fixable if you send us an eeprom dump\n"
+			"(see the demo/eeprom_dump program).\n");
+		reference_in_volts = 5.0;
+	}
+
+	memset(&insn, 0, sizeof(insn));
+	insn.insn = INSN_READ;
+	insn.n = 1;
+	insn.subdev = setup->ad_subdev;
+	insn.data = &data;
+	insn.chanspec = CR_PACK(0, 0, AREF_GROUND) | CR_ALT_SOURCE;
+	retval = comedi_do_insn(setup->dev, &insn);
+	assert(retval >= 0);
+
+	range = comedi_get_range(setup->dev, setup->ad_subdev, 0, 0);
+	assert( range );
+	reference_unitless = comedi_to_phys(data, range, maxdata);
+
+	slope = (reference_unitless - ni67xx_unitless_adc_offset) / reference_in_volts;
+
+	return slope;
+}
+
+/* calibration adc uses RANGE_UNKNOWN, so it will return a value from
+   0.0 to 1.0 instead of a voltage, so we need to renormalize. */
+static void ni67xx_set_target( calibration_setup_t *setup, int obs, double target, double slope)
+{
+	set_target(setup, obs, target);
+	/* convert target from volts to interval [0,1) which calibration
+	 * adc returns */
+	setup->observables[obs].target *= slope;
+	setup->observables[obs].target += ni67xx_unitless_adc_offset;
+}
+
+static void ni67xx_setup_observables( calibration_setup_t *setup )
+{
+	comedi_insn tmpl, po_tmpl;
+	observable *o;
+	int num_ao_channels;
+	int i;
+	double slope;
+
+	slope = ni67xx_unitless_adc_slope(setup);
+
+	/* calibration adc is very slow (15HZ) but accurate, so only sample a few times */
+	setup->sv_order = 1;
+
+	num_ao_channels = comedi_get_n_channels(setup->dev, setup->da_subdev);
+	assert(num_ao_channels >= 0);
+
+	memset( &tmpl, 0, sizeof(tmpl) );
+	tmpl.insn = INSN_READ;
+	tmpl.n = 1;
+	tmpl.subdev = setup->ad_subdev;
+
+	memset( &po_tmpl, 0, sizeof(po_tmpl) );
+	po_tmpl.insn = INSN_WRITE;
+	po_tmpl.n = 1;
+	po_tmpl.subdev = setup->da_subdev;
+
+	setup->n_observables = 0;
+
+	for(i = 0; i < num_ao_channels; i++)
+	{
+		o = setup->observables + ni67xx_ao_ground_observable_index( setup,
+			i, 0);
+		o->reference_source = -1;
+		assert( o->name == NULL );
+		asprintf(&o->name, "dac%i ground, ground referenced", i);
+		o->preobserve_insn = po_tmpl;
+		o->preobserve_insn.chanspec = CR_PACK(i, 0, AREF_GROUND);
+		o->preobserve_insn.data = o->preobserve_data;
+		o->observe_insn = tmpl;
+		o->observe_insn.chanspec = CR_PACK(i, 0, AREF_GROUND);
+		ni67xx_set_target(setup, ni67xx_ao_ground_observable_index(setup, i, 0), 0.0, slope);
+		setup->n_observables++;
+
+		o = setup->observables + ni67xx_ao_mid_observable_index( setup,
+			i, 0);
+		o->reference_source = -1;
+		assert( o->name == NULL );
+		asprintf(&o->name, "dac%i mid, ground referenced", i);
+		o->preobserve_insn = po_tmpl;
+		o->preobserve_insn.chanspec = CR_PACK(i, 0, AREF_GROUND);
+		o->preobserve_insn.data = o->preobserve_data;
+		o->observe_insn = tmpl;
+		o->observe_insn.chanspec = CR_PACK(i, 0, AREF_GROUND);
+		ni67xx_set_target(setup, ni67xx_ao_mid_observable_index(setup, i, 0), 4.0, slope);
+		setup->n_observables++;
+
+		o = setup->observables + ni67xx_ao_high_observable_index( setup, i, 0);
+		o->reference_source = -1;
+		assert( o->name == NULL );
+		asprintf(&o->name, "dac%i high, ground referenced", i);
+		o->preobserve_insn = po_tmpl;
+		o->preobserve_insn.chanspec = CR_PACK( i, 0, AREF_GROUND );
+		o->preobserve_insn.data = o->preobserve_data;
+		o->observe_insn = tmpl;
+		o->observe_insn.chanspec = CR_PACK(i, 0, AREF_GROUND);
+		ni67xx_set_target(setup, ni67xx_ao_high_observable_index(setup, i, 0), 8.0, slope);
+		setup->n_observables++;
+	}
+
+	return;
+}
+
+static int cal_ni_pci_6711(calibration_setup_t *setup)
+{
+	generic_layout_t layout;
+	
+	if( comedi_get_version_code( setup->dev ) <= COMEDI_VERSION_CODE(0, 7, 69))
+	{
+		DPRINT(0, "WARNING: you need comedi driver version 0.7.69 or later\n"
+		 "for this calibration to work properly\n" );
+	}
+	init_generic_layout( &layout );
+	layout.dac_gain = ni67xx_ao_gain_caldac;
+	layout.dac_linearity = ni67xx_ao_linearity_caldac;
+	layout.dac_offset = ni67xx_ao_offset_caldac;
+	layout.dac_high_observable = ni67xx_ao_high_observable_index;
+	layout.dac_mid_observable = ni67xx_ao_mid_observable_index;
+	layout.dac_ground_observable = ni67xx_ao_ground_observable_index;
+	layout.dac_fractional_tolerance = get_tolerance( setup, setup->da_subdev, 1.0 );
+	return generic_cal_ao(setup, &layout);
+}
+
